/*
 * GenerateImportanceGameTask.java
 *
 * Created on January 30, 2006, 12:28 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package gametheory;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.concurrent.Callable;

/**
 * Task to calculate equilibria to an importance game.
 *
 * @author Stephen P. Ryan
 */
public class GenerateImportanceGameTask implements Callable<Game> {

    /**
     * Game to compute equilibria over.
     */
    private Game g;
    /**
     * Index ensuring the temporary file name sent to Gambit does not conflict
     * with another file name in a parallelized environment.
     */
    private int i;
    /**
     * Total number of instances.  Used to ensure that there are no conflicting
     * file names in a parallelized environment when calling Gambit.
     */
    static private int total = 0;
    /**
     * The index of this particular instance.
     */
    private int me = 0;

    /** Creates a new instance of GenerateImportanceGameTask
     * @param g Game to evaluate.
     * @param i Number of the task, used for generating a temporary file to send to Gambit.
     */
    public GenerateImportanceGameTask(Game g, int i) {
        this.g = g;
        this.i = i;
        me = get();
        add();
    }

    /**
     * Updates total number of tasks.
     */
    private synchronized void add() {
        total++;
    }

    /**
     * Returns the index of this task.  Synchronized so that this it keeps
     * track correctly in a multithreaded machine.
     *
     * @return Index of current task.
     */
    private synchronized int get() {
        return total;
    }

    /**
     * Functionality to call Gambit and compute all the equilibria to the game.
     *
     * @return Updated Game object containing equilibria.
     */
    public Game call() {
        /**
         * Compute utilities of game.
         */
        g.calculateUtilities();
        /**
         * Output game to file in format used by Gambit.
         * NOTE: This format may change over time; check the references for Gambit.
         */
        g.outputGame("c:/gambit/game" + me + i + ".nfg");

        /**
         * Call Gambit using a python script.
         */
        try {
            String tempFileName = "tempFile" + i + ".dat";
            Process pinger = Runtime.getRuntime().exec("cmd /C c:/python24/python c:/gambit/enumphc.py " + tempFileName + " < c:/gambit/game" + me + i + ".nfg");
            /**
             * Read back in the output.
             */
            BufferedReader reader = new BufferedReader(new InputStreamReader(pinger.getInputStream()));
            String line = null;
            ArrayList<double[]> equilibriaList = new ArrayList<double[]>();
            while ((line = reader.readLine()) != null) {
                // each line here is one equilibrium as computed by Gambit
                /**
                 * Parse the equilibria as reported by Gambit.
                 */
                if (line.length() > 1) {
                    if (line.substring(0, 2).equalsIgnoreCase("NE")) {
                        int a = 0;
                        int b = line.indexOf(",", a);  // this is just a "NE"
                        a = b + 1;
                        double[] prob = new double[GameTheoryConstants.NUM_PLAYERS * 2];
                        for (int p = 0; p < 2 * GameTheoryConstants.NUM_PLAYERS - 1; p++) {
                            b = line.indexOf(",", a);
                            prob[p] = Double.parseDouble(line.substring(a, b));
                            a = b + 1;
                        }
                        prob[2 * GameTheoryConstants.NUM_PLAYERS - 1] = Double.parseDouble(line.substring(a));
                        /**
                         * Add the computed equilibrium to the game.
                         */
                        equilibriaList.add(prob);
                    }
                }
            }
            /**
             * Set the equlibrium list in the Game.
             */
            g.setEquilibria(equilibriaList);
        } catch (Exception e) {
            e.printStackTrace();
        }
        /**
         * Return the result.
         */
        return g;
    }
}
