/*
 * GenerateImportanceGameTask.java
 *
 * Created on January 30, 2006, 12:28 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package gametheory;

import JSci.maths.statistics.NormalDistribution;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.concurrent.Callable;

/**
 * Generates a Monte Carlo game, solving all the equilibria.
 *
 * @author Stephen P. Ryan
 */
public class GenerateMonteCarloTask implements Callable<Game> {

    /**
     * Index of game, used for generating filename to call Gambit with.  See
     * GenerateImportanceGameTask for more documentation.
     */
    private int i;
    /**
     * Standard normal distribution.
     */
    private static NormalDistribution normal = new NormalDistribution();

    /** Creates a new instance of GenerateImportanceGameTask
     * @param i Index of game, used for generating filename when calling Gambit.
     */
    public GenerateMonteCarloTask(int i) {
        this.i = i;
    }

    public Game call() {
        Game g = new Game(GameTheoryConstants.NUM_PLAYERS, 2);
        ArrayList<double[]> equilibriaList = new ArrayList<double[]>();
        /**
         * Ensure we don't get any funny situations with even numbers of equilibria.
         */
        while (equilibriaList.size() == 0 || Math.floor(equilibriaList.size() / 2.0) == (equilibriaList.size() / 2.0)) {
            try {
                ArrayList actionList = g.getActionList();
                ArrayList<Jama.Matrix> payoffList = new ArrayList<Jama.Matrix>();
                for (int k = 0; k < actionList.size(); k++) {
                    Jama.Matrix action = (Jama.Matrix) actionList.get(k);
                    Jama.Matrix payoff = new Jama.Matrix(GameTheoryConstants.NUM_PLAYERS, 1);
                    for (int m = 0; m < action.getRowDimension(); m++) {
                        if (action.get(m, 0) == 1) {
                            if (m == 0) {
                                payoff.set(m, 0, GameTheoryConstants.BID_COST1 + 2 * Math.random() + normal.inverse(Math.random()));
                            }
                            if (m == 1) {
                                payoff.set(m, 0, GameTheoryConstants.BID_COST2 + 2 * Math.random() + normal.inverse(Math.random()));
                            }
                            if (m == 2) {
                                payoff.set(m, 0, GameTheoryConstants.BID_COST3 + 2 * Math.random() + normal.inverse(Math.random()));
                            }
                            if (m == 3) {
                                payoff.set(m, 0, GameTheoryConstants.BID_COST4 + 2 * Math.random() + normal.inverse(Math.random()));
                            }
                        }
                    }
                    payoffList.add(payoff);
                }
                g.setPayoffList(payoffList);

                g.calculateUtilities();  // this is imposing that the calculateUtilitiescosts are equal to GameTheoryConstants.BID_COST_MONTE_CARLO
                g.outputGame("c:/gambit/game" + i + ".nfg");

                String tempFileName = "tempFile" + i + ".dat";
                Process pinger = Runtime.getRuntime().exec("cmd /C c:/python24/python c:/gambit/enumphc.py " + tempFileName + " < c:/gambit/game" + i + ".nfg");

                BufferedReader reader = new BufferedReader(new InputStreamReader(pinger.getInputStream()));
                String line = null;
                equilibriaList = new ArrayList<double[]>();
                while ((line = reader.readLine()) != null) {
                    // each line here is one equilibrium as computed by Gambit
                    if (line.length() > 1) {
                        if (line.substring(0, 2).equalsIgnoreCase("NE")) {
                            int a = 0;
                            int b = line.indexOf(",", a);  // this is just a "NE"
                            a = b + 1;
                            double[] prob = new double[GameTheoryConstants.NUM_PLAYERS * 2];
                            for (int p = 0; p < 2 * GameTheoryConstants.NUM_PLAYERS - 1; p++) {
                                b = line.indexOf(",", a);
                                prob[p] = Double.parseDouble(line.substring(a, b));
                                a = b + 1;
                            }
                            prob[2 * GameTheoryConstants.NUM_PLAYERS - 1] = Double.parseDouble(line.substring(a));
                            equilibriaList.add(prob);
                        }
                    }
                }
                g.setEquilibria(equilibriaList);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return g;
    }
}
