/*
 * GenerateRecursiveList.java
 *
 * Created on December 26, 2005, 4:33 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package gametheory;

import java.util.ArrayList;

/**
 * Generates the full enumeration of all possible outcomes, given a number of players
 * and a number of actions they can choose.  This class recursively calls itself until
 * the entire set of possibilities is exhausted, and then returns an ArrayList.
 *
 * @author Stephen P. Ryan
 */
public class GenerateRecursiveList {

    /**
     * The set of combinations.
     */
    static ArrayList<Jama.Matrix> combinations;
    /**
     * Number of players.
     */
    static int numPlayers;
    /**
     * Number of actions available to all players (symmetric).
     */
    static int numActions;
    /**
     * Current set of outcomes.
     */
    static Jama.Matrix x;
    
    /** Creates a new instance of GenerateRecursiveList
     * @param numPlayers Number of players.
     * @param numActions Number of actions.
     */
    @SuppressWarnings({"static-access", "static-access"})
    public GenerateRecursiveList(int numPlayers, int numActions) {
	combinations = new ArrayList<Jama.Matrix>();  // store Jama.Matrix list in here
	GenerateRecursiveList.numPlayers = numPlayers;
	GenerateRecursiveList.numActions = numActions;
	x = new Jama.Matrix(numPlayers, 1, 0);
	GenerateRecursiveList next = new GenerateRecursiveList(0);
    }
    
    /**
     * Recursive method to construct the set of outcomes.
     *
     * @param currentPlayer Index of the current update.
     */
    public GenerateRecursiveList(int currentPlayer) {
	while(x.get(currentPlayer, 0)<numActions) {
	    if(currentPlayer<numPlayers-1) {
		GenerateRecursiveList next = new GenerateRecursiveList(currentPlayer+1);
	    }
	    else {
		combinations.add(x.copy());
	    }
	    x.set(currentPlayer, 0, x.get(currentPlayer, 0)+1);
	}
	x.set(currentPlayer, 0, 0);
    }
    
    /**
     * Obtain the combinatoric list of outcomes.
     *
     * @return The list of possible combinations of actions.
     */
    public ArrayList<Jama.Matrix> getCombinations() {
	return combinations;
    }
    
}
