/*
 * ReadInData.java
 *
 * Created on January 4, 2006, 4:11 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */
package gametheory;

import java.io.BufferedReader;
import java.io.FileReader;
import java.util.ArrayList;

/**
 * Reads in data from pre-calculated profit file.  The profit file contains the
 * expected profit for each firm, conditonal on the full vector of entry choices.
 * Further details can be found in Bajari, Hong, and Ryan (2009).
 *
 * @author Stephen P. Ryan
 */
public class ReadInData {

    /**
     * Game list holding observations.
     */
    private ArrayList<Game> gameList = new ArrayList<Game>();

    /** Creates a new instance of ReadInData */
    public ReadInData() {
        ArrayList<ArrayList<Double>> lineList = new ArrayList<ArrayList<Double>>();
        try {
            BufferedReader in = new BufferedReader(new FileReader("profits.txt"));
            String line = in.readLine();
            int counter = 0;
            while (line != null) {
                if (line != null) {
                    ArrayList<Double> numbers = new ArrayList<Double>();
                    // System.out.println(line);
                    // 75
                    // 104
                    // 135
                    // 147
                    // 75 104
                    // 75 135
                    // 75 147
                    // 75	104	135
                    // 75	104	147
                    // 75	135	147
                    // 75	104	135	147
                    // 104	75
                    // 104	135
                    // 104	147
                    // 104	75	135
                    // 104	75	147
                    // 104	135	147
                    // 104	75	135	147
                    // 135	75
                    // 135 104
                    // 135 147
                    // 135	75	104
                    // 135	75	147
                    // 135	104	147
                    // 135	75	104	147
                    // 147	75
                    // 147 104
                    // 147 135
                    // 147	75	104
                    // 147	75	135
                    // 147	104	135
                    // 147	75	104	135
                    // the data are organized in that order
                    // 409 contracts to read in
                    // first thing to do is figure out how to read in all these numbers correctly
                    int a = 0;
                    int b = 0;
                    while (b < line.length()) {
                        // find extent of this number
                        b = a + 1;
                        while (!line.substring(a, b).contains(" ") && !line.substring(a, b).contains("\t") && b < line.length()) {
                            b++;
                        }
                        if (b < line.length() - 1) {
                            b--;
                        }
                        numbers.add(Double.parseDouble(line.substring(a, b)));
                        // System.out.println("number: "+line.substring(a, b));
                        // find extent of whitespaces
                        a = b + 1;
                        if (b < line.length() - 1) {
                            while (line.substring(b, b + 1).contains(" ") || line.substring(b, b + 1).contains("\t") && a < line.length()) {
                                b++;
                                // System.out.println(b);
                            }
                        }
                        // System.out.println("whitespace: "+line.substring(a, b)+" a: "+a+" b: "+b);
                        a = b;
                    }
                    // System.out.println(line.substring(0, a));
                    line = in.readLine();
                    lineList.add(numbers);
                }
            }
            in.close();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(0);
        }

        // so now i have a lineList composed of arraylists of all the numbers in each line
        // next step is to filter out all those lines with zero fringe bidders
        ArrayList<ArrayList<Double>> culledLineList = new ArrayList<ArrayList<Double>>();
        // System.out.println("Line list size: "+lineList.size());
        int removeCount = 0;
        for (int i = 0; i < lineList.size(); i++) {
            boolean remove = false;
            ArrayList<Double> numbers = lineList.get(i);
            for (int j = 0; j < numbers.size(); j++) {
                Double nextNumber = (Double) numbers.get(j);
                if (nextNumber.doubleValue() == 0) {
                    remove = true;
                    removeCount++;
                }
            }
            if (!remove) {
                culledLineList.add(numbers);
            }
        }
        // System.out.println("Culled line list size: "+culledLineList.size());
        // System.out.println("Removed: "+removeCount);
        lineList = null;

        // now go through and assign each of those payoffs to its corresponding game
        // System.out.println("Payoff configurations: "+culledLineList.size()/409.0);

        for (int i = 0; i < 409; i++) {
            gameList.add(new Game(4, 2));
        }

        int i = 0;
        int gameIndex = 0;
        while (i < culledLineList.size()) {
            ArrayList<Double> numbers = culledLineList.get(i);
            // there will be 409 of these in the same configuration
            // and 32 of those configurations
            // so figure out which configuration you are in
            double firmIndex = (Double) numbers.get(0);  // the first number is the index of the firm that we calculated payoffs for
            if (firmIndex == 75) {
                firmIndex = 0;
            }
            if (firmIndex == 104) {
                firmIndex = 1;
            }
            if (firmIndex == 135) {
                firmIndex = 2;
            }
            if (firmIndex == 147) {
                firmIndex = 3;
            }
            // the minimum number of numbers is 3, if the firm faces no major competitors
            // so we can figure out the number of major competitors by subtracting 3 from the size of numbers
            int numberMajorCompetitors = numbers.size() - 3;
            double[] majorCompetitorIDs = new double[numberMajorCompetitors];
            for (int j = 0; j < numberMajorCompetitors; j++) {
                majorCompetitorIDs[j] = numbers.get(j + 1);
                if (majorCompetitorIDs[j] == 75) {
                    majorCompetitorIDs[j] = 0;
                }
                if (majorCompetitorIDs[j] == 104) {
                    majorCompetitorIDs[j] = 1;
                }
                if (majorCompetitorIDs[j] == 135) {
                    majorCompetitorIDs[j] = 2;
                }
                if (majorCompetitorIDs[j] == 147) {
                    majorCompetitorIDs[j] = 3;
                }
            }
            double numberFringeCompetitors = numbers.get(numberMajorCompetitors + 1);
            double expectedPayoffs = numbers.get(numberMajorCompetitors + 2) * 1000;
            // now, just have to figure out how to assign these to the 409 games
            // assign bidder IDs 75, 104, 135, and 147 to player 1, 2, 3, and 4, respectively
            // so game can assign payoffs if you pass it a Jama.Matrix with actions coded as 1's and 0's, the identity of the agent getting the payoff assigned, and the payoff
            Jama.Matrix action = new Jama.Matrix(4, 1);
            action.set((int) firmIndex, 0, 1);
            for (int j = 0; j < numberMajorCompetitors; j++) {
                action.set((int) majorCompetitorIDs[j], 0, 1);
            }
            Game g = gameList.get(gameIndex);
            boolean report = false;
            g.setPayoff(action, (int) firmIndex, expectedPayoffs, report);
            i++;
            gameIndex++;
            if (gameIndex == 409) {
                gameIndex = 0;
            }
        }
    }

    /**
     * Returns game list.
     *
     * @return Game list.
     */
    public ArrayList<Game> getGameList() {
        return gameList;
    }

    /**
     * Sets the game list.
     *
     * @param gameList Game list to be populated.
     */
    public void setGameList(ArrayList<Game> gameList) {
        this.gameList = gameList;
    }
}
