package newtonSolver;

/**
 * Interface for classes that wish to use the Uncmin_f77 optimizer.  User must supply the f_to_minimize routine, and may supply
 * gradient and Hessian methods if desired.
 * 
 * @author Steve Verrill (commented by Stephen P. Ryan)
 */
public interface Uncmin_methods {

    /**
     * Method containing objective function to be optimized.
     * @param x Vector of parameters.  Note that this package uses Fortran indexing, in that the vector starts counting from 1, not 0, so the programmer must take care to ensure tha the indices are computed correctly.
     * @return The value of the objective function evaluated at x.
     */
    double f_to_minimize(double x[]);

   /**
    * User-supplied gradients.
    * @param x Vector of parameters, using Fortran indexing.
    * @param g A vector of empty values which to be filled in by this method
    */
   void gradient(double x[], double g[]);

   /**
    * User-supplied Hessian.
    * @param x Vector of parameters, using Fortran indexing.
    * @param h A matrix of empty values which to be filled in by this method
    */
   void hessian(double x[], double h[][]);

}
